﻿using System.Runtime.InteropServices;

namespace GCBM.tools.Interop;

/// <summary>
///     Represents a single CD/DVD type device, and enables many common operations via a simplified API.
/// </summary>
[ComImport]
[TypeLibType(TypeLibTypeFlags.FDual | TypeLibTypeFlags.FDispatchable | TypeLibTypeFlags.FNonExtensible)]
[Guid("27354133-7F64-5B0F-8F00-5D77AFBE261E")]
public interface IDiscRecorder2
{
    // Ejects the media (if any) and opens the tray
    [DispId(0x100)]
    void EjectMedia();

    // Close the media tray and load any media in the tray.
    [DispId(0x101)]
    void CloseTray();

    // Acquires exclusive access to device.  May be called multiple times.
    [DispId(0x102)]
    void AcquireExclusiveAccess(bool force, string clientName);

    // Releases exclusive access to device.  Call once per AcquireExclusiveAccess().
    [DispId(0x103)]
    void ReleaseExclusiveAccess();

    // Disables Media Change Notification (MCN).
    [DispId(260)]
    void DisableMcn();

    // Re-enables Media Change Notification after a call to DisableMcn()
    [DispId(0x105)]
    void EnableMcn();

    // Initialize the recorder, opening a handle to the specified recorder.
    [DispId(0x106)]
    void InitializeDiscRecorder(string recorderUniqueId);

    // The unique ID used to initialize the recorder.
    [DispId(0)] string ActiveDiscRecorder { get; }

    // The vendor ID in the device's INQUIRY data.
    [DispId(0x201)] string VendorId { get; }

    // The Product ID in the device's INQUIRY data.
    [DispId(0x202)] string ProductId { get; }

    // The Product Revision in the device's INQUIRY data.
    [DispId(0x203)] string ProductRevision { get; }

    // Get the unique volume name (this is not a drive letter).
    [DispId(0x204)] string VolumeName { get; }

    // Drive letters and NTFS mount points to access the recorder.
    [DispId(0x205)] object[] VolumePathNames { [DispId(0x205)] get; }

    // One of the volume names associated with the recorder.
    [DispId(0x206)] bool DeviceCanLoadMedia { [DispId(0x206)] get; }

    // Gets the legacy 'device number' associated with the recorder.  This number is not guaranteed to be static.
    [DispId(0x207)] int LegacyDeviceNumber { [DispId(0x207)] get; }

    // Gets a list of all feature pages supported by the device
    [DispId(520)] object[] SupportedFeaturePages { [DispId(520)] get; }

    // Gets a list of all feature pages with 'current' bit set to true
    [DispId(0x209)] object[] CurrentFeaturePages { [DispId(0x209)] get; }

    // Gets a list of all profiles supported by the device
    [DispId(0x20a)] object[] SupportedProfiles { [DispId(0x20a)] get; }

    // Gets a list of all profiles with 'currentP' bit set to true
    [DispId(0x20b)] object[] CurrentProfiles { [DispId(0x20b)] get; }

    // Gets a list of all MODE PAGES supported by the device
    [DispId(0x20c)] object[] SupportedModePages { [DispId(0x20c)] get; }

    // Queries the device to determine who, if anyone, has acquired exclusive access
    [DispId(0x20d)] string ExclusiveAccessOwner { [DispId(0x20d)] get; }
}